#!/usr/bin/env python3
"""A test module with various utilities."""


def utility_function():
    """A simple utility function."""
    return "utility result"


class TestClass:
    """A test class."""

    def __init__(self, value):
        self.value = value

    def get_value(self):
        return self.value


# Some constants
CONSTANT_VALUE = 42
