"""Utility module for testing regular import aliases."""


def helper_function():
    """A simple helper function."""
    return "helper_result"


class UtilityClass:
    """A simple utility class."""

    def __init__(self, value):
        self.value = value

    def get_value(self):
        return self.value
