#!/usr/bin/env python3
# Generated by Serpen - Python Source Bundler
# https://github.com/tinovyatkin/serpen

# Preserved imports
import collections
import math
import os
import typing

# ─ Entry Module: main ─
# Test file with unused imports
import os  # Used
import math  # Used


def main():
    # Use os
    print(f"Current directory: {os.getcwd()}")
    # Use math
    result = math.sqrt(16)
    # Use List from typing
    numbers: List[int] = [1, 2, 3, 4]
    # Use defaultdict
    dd = defaultdict(int)
    dd["test"] = 5
    print(f"Square root: {result}")
    print(f"Numbers: {numbers}")
    print(f"Defaultdict: {dict(dd)}")


if __name__ == "__main__":
    main()
