"""
Constants module with global variables.
These will conflict with config.py when bundled.
"""

# API configuration
API_URL = "https://api.constants.com"
VERSION = "1.0.0"
DEFAULT_TIMEOUT = 30

# Additional constants
MAX_CONNECTIONS = 100
RETRY_DELAY = 5
