#!/usr/bin/env python3
# Generated by Serpen - Python Source Bundler
# https://github.com/tinovyatkin/serpen

# Preserved imports
import math
import os

# ─ Entry Module: simple_test ─
# Simple test with completely used or unused imports
import os  # Used
import math  # Used


def main():
    # Use os
    print(f"Current directory: {os.getcwd()}")

    # Use math
    result = math.sqrt(16)
    print(f"Square root: {result}")


if __name__ == "__main__":
    main()
